@@ -5,11 +5,11 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2017, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2015, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
- * are also available at https://curl.haxx.se/docs/copyright.html.
+ * are also available at http://curl.haxx.se/docs/copyright.html.
  *
  * You may opt to use, copy, modify, merge, publish, distribute and/or sell
  * copies of the Software, and permit persons to whom the Software is
@@ -45,7 +45,7 @@ void dump(const char *text, unsigned char *ptr, size_t size,
   size_t i;
   size_t c;
 
-  unsigned int width = 0x10;
+  unsigned int width=0x10;
 
   if(nohex)
     /* without the hex output, we can fit more on screen */
@@ -54,32 +54,30 @@ void dump(const char *text, unsigned char *ptr, size_t size,
   fprintf(stderr, "%s, %ld bytes (0x%lx)\n",
           text, (long)size, (long)size);
 
-  for(i = 0; i<size; i += width) {
+  for(i=0; i<size; i+= width) {
 
     fprintf(stderr, "%4.4lx: ", (long)i);
 
     if(!nohex) {
       /* hex not disabled, show it */
       for(c = 0; c < width; c++)
-        if(i + c < size)
-          fprintf(stderr, "%02x ", ptr[i + c]);
+        if(i+c < size)
+          fprintf(stderr, "%02x ", ptr[i+c]);
         else
           fputs("   ", stderr);
     }
 
-    for(c = 0; (c < width) && (i + c < size); c++) {
+    for(c = 0; (c < width) && (i+c < size); c++) {
       /* check for 0D0A; if found, skip past and start a new line of output */
-      if(nohex && (i + c + 1 < size) && ptr[i + c] == 0x0D &&
-         ptr[i + c + 1] == 0x0A) {
-        i += (c + 2 - width);
+      if (nohex && (i+c+1 < size) && ptr[i+c]==0x0D && ptr[i+c+1]==0x0A) {
+        i+=(c+2-width);
         break;
       }
       fprintf(stderr, "%c",
-              (ptr[i + c] >= 0x20) && (ptr[i + c]<0x80)?ptr[i + c]:'.');
+              (ptr[i+c]>=0x20) && (ptr[i+c]<0x80)?ptr[i+c]:'.');
       /* check again for 0D0A, to avoid an extra \n if it's at width */
-      if(nohex && (i + c + 2 < size) && ptr[i + c + 1] == 0x0D &&
-         ptr[i + c + 2] == 0x0A) {
-        i += (c + 3 - width);
+      if (nohex && (i+c+2 < size) && ptr[i+c+1]==0x0D && ptr[i+c+2]==0x0A) {
+        i+=(c+3-width);
         break;
       }
     }
@@ -95,10 +93,9 @@ int my_trace(CURL *handle, curl_infotype type,
   const char *text;
   (void)handle; /* prevent compiler warning */
   (void)userp;
-  switch(type) {
+  switch (type) {
   case CURLINFO_TEXT:
     fprintf(stderr, "== Info: %s", data);
-    /* FALLTHROUGH */
   default: /* in case a new one is introduced to shock us */
     return 0;
 
@@ -126,11 +123,9 @@ int my_trace(CURL *handle, curl_infotype type,
   return 0;
 }
 
-#define OUTPUTFILE "dl"
-
 static void setup(CURL *hnd)
 {
-  FILE *out = fopen(OUTPUTFILE, "wb");
+  FILE *out = fopen("dl", "wb");
 
   /* write to this file */
   curl_easy_setopt(hnd, CURLOPT_WRITEDATA, out);
@@ -138,7 +133,7 @@ static void setup(CURL *hnd)
   /* set the same URL */
   curl_easy_setopt(hnd, CURLOPT_URL, "https://localhost:8443/index.html");
 
-  /* please be verbose */
+  /* send it verbose for max debuggaility */
   curl_easy_setopt(hnd, CURLOPT_VERBOSE, 1L);
   curl_easy_setopt(hnd, CURLOPT_DEBUGFUNCTION, my_trace);
 
@@ -172,7 +167,7 @@ static int server_push_callback(CURL *parent,
 
   (void)parent; /* we have no use for this */
 
-  snprintf(filename, 128, "push%u", count++);
+  sprintf(filename, "push%u", count++);
 
   /* here's a new stream, save it in a new file for each new push */
   out = fopen(filename, "wb");
@@ -183,14 +178,14 @@ static int server_push_callback(CURL *parent,
   fprintf(stderr, "**** push callback approves stream %u, got %d headers!\n",
           count, (int)num_headers);
 
-  for(i = 0; i<num_headers; i++) {
+  for(i=0; i<num_headers; i++) {
     headp = curl_pushheader_bynum(headers, i);
     fprintf(stderr, "**** header %u: %s\n", (int)i, headp);
   }
 
   headp = curl_pushheader_byname(headers, ":path");
   if(headp) {
-    fprintf(stderr, "**** The PATH is %s\n", headp /* skip :path + colon */);
+    fprintf(stderr, "**** The PATH is %s\n", headp /* skip :path + colon */ );
   }
 
   (*transfers)++; /* one more */
@@ -206,7 +201,7 @@ int main(void)
   CURL *easy;
   CURLM *multi_handle;
   int still_running; /* keep number of running handles */
-  int transfers = 1; /* we start with one */
+  int transfers=1; /* we start with one */
   struct CURLMsg *m;
 
   /* init a multi stack */
@@ -283,7 +278,7 @@ int main(void)
     else {
       /* Note that on some platforms 'timeout' may be modified by select().
          If you need access to the original value save a copy beforehand. */
-      rc = select(maxfd + 1, &fdread, &fdwrite, &fdexcep, &timeout);
+      rc = select(maxfd+1, &fdread, &fdwrite, &fdexcep, &timeout);
     }
 
     switch(rc) {
